﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace TextMorphCheckBoxDemo
{
    public partial class InterpolatedPolylineControl : UserControl
    {
        public static readonly DependencyProperty Points1Property =
            DependencyProperty.Register("Points1",
                typeof(PointCollection),
                typeof(InterpolatedPolylineControl),
                new PropertyMetadata(OnPoints1Changed));

        public static readonly DependencyProperty Points2Property =
            DependencyProperty.Register("Points2",
                typeof(PointCollection),
                typeof(InterpolatedPolylineControl),
                new PropertyMetadata(OnPoints2Changed));

        public static readonly DependencyProperty ProgressProperty =
            DependencyProperty.Register("Progress",
                typeof(double),
                typeof(InterpolatedPolylineControl),
                new PropertyMetadata(0.0, OnProgressChanged));

        public static readonly DependencyProperty TransformProperty =
            DependencyProperty.Register("Transform",
                typeof(Transform),
                typeof(InterpolatedPolylineControl),
                new PropertyMetadata(OnTransformChanged));

        public static readonly DependencyProperty StrokeProperty =
            DependencyProperty.Register("Stroke",
                typeof(Brush),
                typeof(InterpolatedPolylineControl),
                new PropertyMetadata(OnStrokeChanged));

        public static readonly DependencyProperty StrokeThicknessProperty =
            DependencyProperty.Register("StrokeThickness",
                typeof(double),
                typeof(InterpolatedPolylineControl),
                new PropertyMetadata(1.0, OnStrokeThicknessChanged));

        public InterpolatedPolylineControl()
        {
            InitializeComponent();
        }

        public PointCollection Points1
        {
            set { SetValue(Points1Property, value); }
            get { return (PointCollection)GetValue(Points1Property); }
        }
        public PointCollection Points2
        {
            set { SetValue(Points2Property, value); }
            get { return (PointCollection)GetValue(Points2Property); }
        }

        public double Progress
        {
            set { SetValue(ProgressProperty, value); }
            get { return (double)GetValue(ProgressProperty); }
        }

        public Transform Transform
        {
            set { SetValue(TransformProperty, value); }
            get { return (Transform)GetValue(TransformProperty); }
        }

        public Brush Stroke
        {
            set { SetValue(StrokeProperty, value); }
            get { return (Brush)GetValue(StrokeProperty); }
        }

        public double StrokeThickness
        {
            set { SetValue(StrokeThicknessProperty, value); }
            get { return (double)GetValue(StrokeThicknessProperty); }
        }

        static void OnPoints1Changed(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as InterpolatedPolylineControl).OnPoints1Changed(args);
        }

        void OnPoints1Changed(DependencyPropertyChangedEventArgs args)
        {
            (Resources["interpolatorKey"] as PointCollectionInterpolator).Points1 = Points1;
        }

        static void OnPoints2Changed(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as InterpolatedPolylineControl).OnPoints2Changed(args);
        }

        void OnPoints2Changed(DependencyPropertyChangedEventArgs args)
        {
            (Resources["interpolatorKey"] as PointCollectionInterpolator).Points2 = Points2;
        }

        static void OnProgressChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as InterpolatedPolylineControl).OnProgressChanged(args);
        }

        void OnProgressChanged(DependencyPropertyChangedEventArgs args)
        {
            (Resources["interpolatorKey"] as PointCollectionInterpolator).Progress = Progress;
        }

        static void OnTransformChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as InterpolatedPolylineControl).OnTransformChanged(args);
        }

        void OnTransformChanged(DependencyPropertyChangedEventArgs args)
        {
            (Resources["interpolatorKey"] as PointCollectionInterpolator).Transform = Transform;
        }

        static void OnStrokeChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as InterpolatedPolylineControl).polyline.Stroke = args.NewValue as Brush;
        }

        static void OnStrokeThicknessChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as InterpolatedPolylineControl).polyline.StrokeThickness = (double)args.NewValue;
        }
    }
}
